@@ -279,7 +279,7 @@ xpress_init_output(struct xpress_output_bitstream *os, void *buffer, size_t size
  * If the output buffer space is exhausted, then the bits will be ignored, and
  * xpress_flush_output() will return 0 when it gets called.
  */
-static inline void
+static forceinline void
 xpress_write_bits(struct xpress_output_bitstream *os,
 		  const u32 bits, const unsigned num_bits)
 {
@@ -303,7 +303,7 @@ xpress_write_bits(struct xpress_output_bitstream *os,
 /*
  * Interweave a literal byte into the output bitstream.
  */
-static inline void
+static forceinline void
 xpress_write_byte(struct xpress_output_bitstream *os, u8 byte)
 {
 	if (os->next_byte < os->end)
@@ -313,7 +313,7 @@ xpress_write_byte(struct xpress_output_bitstream *os, u8 byte)
 /*
  * Interweave two literal bytes into the output bitstream.
  */
-static inline void
+static forceinline void
 xpress_write_u16(struct xpress_output_bitstream *os, u16 v)
 {
 	if (os->end - os->next_byte >= 2) {
@@ -338,7 +338,7 @@ xpress_flush_output(struct xpress_output_bitstream *os)
 	return os->next_byte - os->start;
 }
 
-static inline void
+static forceinline void
 xpress_write_extra_length_bytes(struct xpress_output_bitstream *os,
 				unsigned adjusted_len)
 {
@@ -353,7 +353,7 @@ xpress_write_extra_length_bytes(struct xpress_output_bitstream *os,
 }
 
 /* Output a match or literal.  */
-static inline void
+static forceinline void
 xpress_write_item(struct xpress_item item, struct xpress_output_bitstream *os,
 		  const u32 codewords[], const u8 lens[])
 {
@@ -413,7 +413,7 @@ xpress_write_item_list(struct xpress_output_bitstream *os,
 			unsigned sym;
 
 			adjusted_len = length - XPRESS_MIN_MATCH_LEN;
-			log2_offset = fls32(offset);
+			log2_offset = bsr32(offset);
 			len_hdr = min(0xF, adjusted_len);
 			sym = XPRESS_NUM_CHARS + ((log2_offset << 4) | len_hdr);
 
@@ -484,7 +484,7 @@ xpress_write(struct xpress_compressor *c, void *out, size_t out_nbytes_avail,
 
 /* Tally the Huffman symbol for a literal and return the intermediate
  * representation of that literal.  */
-static inline struct xpress_item
+static forceinline struct xpress_item
 xpress_record_literal(struct xpress_compressor *c, unsigned literal)
 {
 	c->freqs[literal]++;
@@ -496,12 +496,12 @@ xpress_record_literal(struct xpress_compressor *c, unsigned literal)
 
 /* Tally the Huffman symbol for a match and return the intermediate
  * representation of that match.  */
-static inline struct xpress_item
+static forceinline struct xpress_item
 xpress_record_match(struct xpress_compressor *c, unsigned length, unsigned offset)
 {
 	unsigned adjusted_len = length - XPRESS_MIN_MATCH_LEN;
 	unsigned len_hdr = min(adjusted_len, 0xF);
-	unsigned log2_offset = fls32(offset);
+	unsigned log2_offset = bsr32(offset);
 	unsigned sym = XPRESS_NUM_CHARS + ((log2_offset << 4) | len_hdr);
 
 	c->freqs[sym]++;
@@ -755,7 +755,7 @@ xpress_tally_item_list(struct xpress_compressor *c,
 			unsigned sym;
 
 			adjusted_len = length - XPRESS_MIN_MATCH_LEN;
-			log2_offset = fls32(offset);
+			log2_offset = bsr32(offset);
 			len_hdr = min(0xF, adjusted_len);
 			sym = XPRESS_NUM_CHARS + ((log2_offset << 4) | len_hdr);
 
@@ -831,7 +831,7 @@ xpress_find_min_cost_path(struct xpress_compressor *c, size_t in_nbytes,
 				u32 offset_cost;
 
 				offset = match->offset;
-				log2_offset = fls32(offset);
+				log2_offset = bsr32(offset);
 				offset_cost = log2_offset;
 				do {
 					unsigned len_hdr;
@@ -860,7 +860,7 @@ xpress_find_min_cost_path(struct xpress_compressor *c, size_t in_nbytes,
 				u32 offset_cost;
 
 				offset = match->offset;
-				log2_offset = fls32(offset);
+				log2_offset = bsr32(offset);
 				offset_cost = log2_offset;
 				do {
 					unsigned adjusted_len;
@@ -964,7 +964,6 @@ xpress_find_matches(struct xpress_compressor * restrict c,
 				bt_matchfinder_skip_position(&c->bt_mf,
 							     in_begin,
 							     in_next - in_begin,
-							     max_len,
 							     nice_len,
 							     c->max_search_depth,
 							     next_hashes);
